﻿// Copyright 2007-2012 Chris Patterson, Dru Sellers, Travis Smith, et. al.
//  
// Licensed under the Apache License, Version 2.0 (the "License"); you may not use 
// this file except in compliance with the License. You may obtain a copy of the 
// License at 
// 
//     http://www.apache.org/licenses/LICENSE-2.0 
// 
// Unless required by applicable law or agreed to in writing, software distributed 
// under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR 
// CONDITIONS OF ANY KIND, either express or implied. See the License for the 
// specific language governing permissions and limitations under the License.
namespace Topshelf.HostConfigurators
{
    using System;
    using System.Collections.Generic;
    using Builders;
    using Configurators;

    /// <summary>
    /// Adds a dependency to the InstallBuilder (ignored otherwise)
    /// </summary>
    public class DependencyHostConfigurator :
        HostBuilderConfigurator
    {
        readonly string _name;

        public DependencyHostConfigurator(string name)
        {
            if (name == null)
                throw new ArgumentNullException("name");

            _name = name;
        }

        public IEnumerable<ValidateResult> Validate()
        {
            if (string.IsNullOrEmpty(_name))
                yield return this.Failure("Dependency", "must not be null");
        }

        public HostBuilder Configure(HostBuilder builder)
        {
            if (builder == null)
                throw new ArgumentNullException("builder");

            builder.Match<InstallBuilder>(x => x.AddDependency(_name));

            return builder;
        }
    }
}